;IR Remote Control (NEC-Kodierung) mit seriellem Interface V1.0
;(c)2010 J.KORDAS jkordas(at)web.de
;----------------------------------------------------------------------------------
.EQU	VERTAG	=	20
.EQU	VERMON	=	8
.EQU	VERJHR	=	10
;----------------------------------------------------------------------------------

.include "m48def.inc"			;ATMEGA48 (7.3728 MHz)

;PROG.PARAMETER: BODLEVEL: 101 (2,7V), BOOTSZ: 00, CKSEL: 0111, SUT: 11

;PORT-Belegung:	INP-B0:			OUT-C0: DUO-LED		INP-D0: RXD
;(Low-aktiv)	I/O-B1: IR-LED		OUT-C1: DUO-LED		OUT-D1: TXD
;		INP-B2: 		INP-C2: 		INP-D2:
;		INP-B3: 		INP-C3: 		INP-D3:
;		INP-B4: 		INP-C4: 		INP-D5:
;		INP-B5: 		INP-C5: 		INP-D6:
;					INP-C6: 		INP-D7:
								

.DEF	SIC	=	r8		;SYSINT COUNTER
.DEF	TSREG	=	r9		;SREG TEMP

.EQU	NCODE	=	1		;NET-CODE
.EQU	VCMD	=	$ff		;QUEUE VOID-CMD CODE
.EQU	PWMFREQ	=	193		;PWM FREQ 38 KHZ
.EQU	PWMDZ	=	130		;PWM FREQ 38 KHZ DUTY CYCLE
.EQU	RRTIME	=	3		;RECEIVER RESET TIME (HSEK)
.EQU	CPTIME	=	100		;COMMAND PAUSE TIME (HSEK)
.EQU	TIP	=	7		;GPIOR0BIT: TRANSMISSION IN PROGRESS
.EQU	RIP	=	6		;GPIOR0BIT: RECEPTION IN PROGRESS

;----------------------------------------------------------------------------------
;Systeminterruptvektor-Tabelle
			
		rjmp	RESET		;0000: 	Reset-Vektor
		rjmp	RETURN		;0001: 	INT00 IRQ0
		rjmp	RETURN		;0002: 	INT01 IRQ1
		rjmp	RETURN		;0003: 	INT02 PCINT0
		rjmp	RETURN		;0004: 	INT03 PCINT1
		rjmp	RETURN		;0005: 	INT04 PCINT2
		rjmp	RETURN		;0006: 	INT05 WDT
		rjmp	RETURN		;0007: 	INT06 Timer2 CompareA
		rjmp	RETURN		;0008: 	INT07 Timer2 CompareB
		rjmp	IBTINT		;0009: 	INT08 Timer2 Overflow
		rjmp	RETURN		;000A: 	INT09 Timer1 Capture
		rjmp	RETURN		;000B: 	INT10 Timer1 CompareA
		rjmp	RETURN		;000C: 	INT11 Timer1 CompareB
		rjmp	RETURN		;000D: 	INT12 Timer1 Overflow
		rjmp	SYSINT		;000E: 	INT13 Timer0 CompareA
		rjmp	RETURN		;000F: 	INT14 Timer0 CompareB
		rjmp	RETURN		;0010: 	INT15 Timer0 Overflow
		rjmp	RETURN		;0011: 	INT16 SPI Transfer Complete
		rjmp	RXINT		;0012:	INT17 USART RX Complete
		rjmp	TXINT		;0013:	INT18 USART UDR Empty
		rjmp	RETURN		;0014:	INT19 USART TX Complete
		rjmp	RETURN		;0015:	INT20 ADC Complete
		rjmp	RETURN		;0016:	INT21 EEPROM Ready
		rjmp	RETURN		;0017:	INT22 Analog Comparator
		rjmp	RETURN		;0018:	INT23 2-wire Serial Interface
		rjmp	RETURN		;0019:	INT24 SPM Ready

;----------------------------------------------------------------------------------

RESET:		clr	r16			;DEL SYSRAM
		ldi	YL,low(RAMEND)
		ldi	YH,high(RAMEND)
		ldi	ZL,low(DATARAM)
		ldi	ZH,high(DATARAM)
delram1:	st	Z+,r16
		sbiw	YH:YL,1
		brne	delram1

		ldi	r16,low(COMTBUF)	;RAM-Initialisierung
		sts	COMTBPL,r16
		ldi	r16,low(COMRBUF)
		sts	COMRBPL,r16
		ldi	r16,12
		sts	IRCC,r16

SYSINIT:	sbi	DDRC,0			;Hardware-Initialisierung
		sbi	DDRC,1
		sbi	PORTC,0			;Green LED ON: Bereitschaft
		cbi	PORTC,1

		ldi	r16,$23			;SYSINT-Init (Timer0 Compare INT)
		out	OCR0A,r16		;7,3728 MHz / (SYSINT-Teiler 35+1
		ldi	r16,$05			;               * Vorteiler 1024)
		out	TCCR0B,r16		;			 = 200 Hz
		ldi	r16,$02				
		out	TCCR0A,r16		;SET CTC-MODE
		sts	TIMSK0,r16		;Enable Timer0 Compare-INT

		clr	r16			;Init 38KHZ 1:2 PWM
		ldi	r17,PWMFREQ
		sts	ICR1H,r16
		sts	ICR1L,r17
		ldi	r17,PWMDZ
		sts	OCR1AH,r16
		sts	OCR1AL,r17
		ldi	r16,$c2
		sts	TCCR1A,r16
		ldi	r16,$19
		sts	TCCR1B,r16

		clr	r16			;Konfiguration USART
		ldi	r17,47			;Baudrate= 9600 (47)
		sts	UBRR0H,r16
		sts	UBRR0L,r17		;Set Baudrate
		ldi	r16,$6			;8Data/1Stop/No Parity asynchron
		sts	UCSR0C,r16		;Set Configuration
		ldi	r16,$98			;Transmitter/Receiver+INT
		sts	UCSR0B,r16		;Enable
		sei				;Systeminterrupts starten

		ldi 	r16,1
		out 	MCUCR,r16       	;Enable sleep mode

mainprg:	sleep				;Main-Program
		rjmp	mainprg			;sleeping ...



;----------------------------------------------------------------------------------

.include "sysint.asm"
.include "sysram.asm"

;----------------------------------------------------------------------------------
;NEC-CODES	Opticum HD-C10 (DEV.ADR. F304)

;Taste		HEX-Code		Taste		HEX-Code

;Power		5F			Rec		1C
;Mute		57			Exit		1D
;Help		11			Info		06
;TVformat	07			EPG		09
;TV/Radio	14			AUF		18
;Mode		1A			AB		46
;1	        56			LINKS		41
;2	        5A			RECHTS		49
;3	        5E			OK		45
;4	        55			FileList	17
;5	        59			Slow		0D
;6		5D			Prev/Next	02
;7		54			Next		42
;8		58			ROT Audio	5B
;9		5C			GRN Subtitle	1F
;0		1B			GELB Sleep	08
;Menu		4A			BLAU Text	03
;Recall		15			WEISS Fav	00
;Rew		1E			SCHWARZ	Status	0C
;Stop		10			GRAU Option	19
;Play/Pause	12			GRAU Find	16
;FF		01

;NEC IR-Telegramm: DeviceAdrLow (04), DeviceAdrHigh (F3), Keycode, Invert-Keycode

